h2. Active Record Query Interface

This guide covers different ways to retrieve data from the database using Active Record. By referring to this guide, you will be able to:

* Find records using a variety of methods and conditions
* Specify the order, retrieved attributes, grouping, and other properties of the found records
* Use eager loading to reduce the number of database queries needed for data retrieval
* Use dynamic finders methods
* Check for the existence of particular records
* Perform various calculations on Active Record models
* Run EXPLAIN on relations

endprologue.

WARNING. This Guide is based on Rails 3.0. Some of the code shown here will not work in other versions of Rails.

If you're used to using raw SQL to find database records, then you will generally find that there are better ways to carry out the same operations in Rails. Active Record insulates you from the need to use SQL in most cases.

Code examples throughout this guide will refer to one or more of the following models:

TIP: All of the following models use +id+ as the primary key, unless specified otherwise.

<ruby>
class Client < ActiveRecord::Base
  has_one :address
  has_many :orders
  has_and_belongs_to_many :roles
end
</ruby>

<ruby>
class Address < ActiveRecord::Base
  belongs_to :client
end
</ruby>

<ruby>
class Order < ActiveRecord::Base
  belongs_to :client, :counter_cache => true
end
</ruby>

<ruby>
class Role < ActiveRecord::Base
  has_and_belongs_to_many :clients
end
</ruby>

Active Record will perform queries on the database for you and is compatible with most database systems (MySQL, PostgreSQL and SQLite to name a few). Regardless of which database system you're using, the Active Record method format will always be the same.

h3. Retrieving Objects from the Database

To retrieve objects from the database, Active Record provides several finder methods. Each finder method allows you to pass arguments into it to perform certain queries on your database without writing raw SQL.

The methods are:
* +where+
* +select+
* +group+
* +order+
* +reorder+
* +reverse_order+
* +limit+
* +offset+
* +joins+
* +includes+
* +lock+
* +readonly+
* +from+
* +having+

All of the above methods return an instance of <tt>ActiveRecord::Relation</tt>.

The primary operation of <tt>Model.find(options)</tt> can be summarized as:

* Convert the supplied options to an equivalent SQL query.
* Fire the SQL query and retrieve the corresponding results from the database.
* Instantiate the equivalent Ruby object of the appropriate model for every resulting row.
* Run +after_find+ callbacks, if any.

h4. Retrieving a Single Object

Active Record provides five different ways of retrieving a single object.

h5. Using a Primary Key

Using <tt>Model.find(primary_key)</tt>, you can retrieve the object corresponding to the specified _primary key_ that matches any supplied options. For example:

<ruby>
# Find the client with primary key (id) 10.
client = Client.find(10)
# => #<Client id: 10, first_name: "Ryan">
</ruby>

The SQL equivalent of the above is:

<sql>
SELECT * FROM clients WHERE (clients.id = 10) LIMIT 1
</sql>

<tt>Model.find(primary_key)</tt> will raise an +ActiveRecord::RecordNotFound+ exception if no matching record is found.

h5. +first+

<tt>Model.first</tt> finds the first record matched by the supplied options, if any. For example:

<ruby>
client = Client.first
# => #<Client id: 1, first_name: "Lifo">
</ruby>

The SQL equivalent of the above is:

<sql>
SELECT * FROM clients LIMIT 1
</sql>

<tt>Model.first</tt> returns +nil+ if no matching record is found. No exception will be raised.

h5. +last+

<tt>Model.last</tt> finds the last record matched by the supplied options. For example:

<ruby>
client = Client.last
# => #<Client id: 221, first_name: "Russel">
</ruby>

The SQL equivalent of the above is:

<sql>
SELECT * FROM clients ORDER BY clients.id DESC LIMIT 1
</sql>

<tt>Model.last</tt> returns +nil+ if no matching record is found. No exception will be raised.

h5(#first_1). +first!+

<tt>Model.first!</tt> finds the first record. For example:

<ruby>
client = Client.first!
# => #<Client id: 1, first_name: "Lifo">
</ruby>

The SQL equivalent of the above is:

<sql>
SELECT * FROM clients LIMIT 1
</sql>

<tt>Model.first!</tt> raises +RecordNotFound+ if no matching record is found.

h5(#last_1). +last!+

<tt>Model.last!</tt> finds the last record. For example:

<ruby>
client = Client.last!
# => #<Client id: 221, first_name: "Russel">
</ruby>

The SQL equivalent of the above is:

<sql>
SELECT * FROM clients ORDER BY clients.id DESC LIMIT 1
</sql>

<tt>Model.last!</tt> raises +RecordNotFound+ if no matching record is found.

h4. Retrieving Multiple Objects

h5. Using Multiple Primary Keys

<tt>Model.find(array_of_primary_key)</tt> accepts an array of _primary keys_, returning an array containing all of the matching records for the supplied _primary keys_. For example:

<ruby>
# Find the clients with primary keys 1 and 10.
client = Client.find([1, 10]) # Or even Client.find(1, 10)
# => [#<Client id: 1, first_name: "Lifo">, #<Client id: 10, first_name: "Ryan">]
</ruby>

The SQL equivalent of the above is:

<sql>
SELECT * FROM clients WHERE (clients.id IN (1,10))
</sql>

WARNING: <tt>Model.find(array_of_primary_key)</tt> will raise an +ActiveRecord::RecordNotFound+ exception unless a matching record is found for <strong>all</strong> of the supplied primary keys.

h4. Retrieving Multiple Objects in Batches

We often need to iterate over a large set of records, as when we send a newsletter to a large set of users, or when we export data.

This may appear straightforward:

<ruby>
# This is very inefficient when the users table has thousands of rows.
User.all.each do |user|
  NewsLetter.weekly_deliver(user)
end
</ruby>

But this approach becomes increasingly impractical as the table size increases, since +User.all.each+ instructs Active Record to fetch _the entire table_ in a single pass, build a model object per row, and then keep the entire array of model objects in memory. Indeed, if we have a large number of records, the entire collection may exceed the amount of memory available.

Rails provides two methods that address this problem by dividing records into memory-friendly batches for processing. The first method, +find_each+, retrieves a batch of records and then yields _each_ record to the block individually as a model. The second method, +find_in_batches+, retrieves a batch of records and then yields _the entire batch_ to the block as an array of models.

TIP: The +find_each+ and +find_in_batches+ methods are intended for use in the batch processing of a large number of records that wouldn't fit in memory all at once. If you just need to loop over a thousand records the regular find methods are the preferred option.

h5. +find_each+

The +find_each+ method retrieves a batch of records and then yields _each_ record to the block individually as a model. In the following example, +find_each+ will retrieve 1000 records (the current default for both +find_each+ and +find_in_batches+) and then yield each record individually to the block as a model. This process is repeated until all of the records have been processed:

<ruby>
User.find_each do |user|
  NewsLetter.weekly_deliver(user)
end
</ruby>

h6. Options for +find_each+

The +find_each+ method accepts most of the options allowed by the regular +find+ method, except for +:order+ and +:limit+, which are reserved for internal use by +find_each+.

Two additional options, +:batch_size+ and +:start+, are available as well.

*+:batch_size+*

The +:batch_size+ option allows you to specify the number of records to be retrieved in each batch, before being passed individually to the block. For example, to retrieve records in batches of 5000:

<ruby>
User.find_each(:batch_size => 5000) do |user|
  NewsLetter.weekly_deliver(user)
end
</ruby>

*+:start+*

By default, records are fetched in ascending order of the primary key, which must be an integer. The +:start+ option allows you to configure the first ID of the sequence whenever the lowest ID is not the one you need. This would be useful, for example, if you wanted to resume an interrupted batch process, provided you saved the last processed ID as a checkpoint.

For example, to send newsletters only to users with the primary key starting from 2000, and to retrieve them in batches of 5000:

<ruby>
User.find_each(:start => 2000, :batch_size => 5000) do |user|
  NewsLetter.weekly_deliver(user)
end
</ruby>

Another example would be if you wanted multiple workers handling the same processing queue. You could have each worker handle 10000 records by setting the appropriate <tt>:start</tt> option on each worker.

NOTE: The +:include+ option allows you to name associations that should be loaded alongside with the models.

h5. +find_in_batches+

The +find_in_batches+ method is similar to +find_each+, since both retrieve batches of records. The difference is that +find_in_batches+ yields _batches_ to the block as an array of models, instead of individually. The following example will yield to the supplied block an array of up to 1000 invoices at a time, with the final block containing any remaining invoices:

<ruby>
# Give add_invoices an array of 1000 invoices at a time
Invoice.find_in_batches(:include => :invoice_lines) do |invoices|
  export.add_invoices(invoices)
end
</ruby>

NOTE: The +:include+ option allows you to name associations that should be loaded alongside with the models.

h6. Options for +find_in_batches+

The +find_in_batches+ method accepts the same +:batch_size+ and +:start+ options as +find_each+, as well as most of the options allowed by the regular +find+ method, except for +:order+ and +:limit+, which are reserved for internal use by +find_in_batches+.

h3. Conditions

The +where+ method allows you to specify conditions to limit the records returned, representing the +WHERE+-part of the SQL statement. Conditions can either be specified as a string, array, or hash.

h4. Pure String Conditions

If you'd like to add conditions to your find, you could just specify them in there, just like +Client.where("orders_count = '2'")+. This will find all clients where the +orders_count+ field's value is 2.

WARNING: Building your own conditions as pure strings can leave you vulnerable to SQL injection exploits. For example, +Client.where("first_name LIKE '%#{params[:first_name]}%'")+ is not safe. See the next section for the preferred way to handle conditions using an array.

h4. Array Conditions

Now what if that number could vary, say as an argument from somewhere? The find would then take the form:

<ruby>
Client.where("orders_count = ?", params[:orders])
</ruby>

Active Record will go through the first element in the conditions value and any additional elements will replace the question marks +(?)+ in the first element.

If you want to specify multiple conditions:

<ruby>
Client.where("orders_count = ? AND locked = ?", params[:orders], false)
</ruby>

In this example, the first question mark will be replaced with the value in +params[:orders]+ and the second will be replaced with the SQL representation of +false+, which depends on the adapter.

This code is highly preferable:

<ruby>
Client.where("orders_count = ?", params[:orders])
</ruby>

to this code:

<ruby>
Client.where("orders_count = #{params[:orders]}")
</ruby>

because of argument safety. Putting the variable directly into the conditions string will pass the variable to the database *as-is*. This means that it will be an unescaped variable directly from a user who may have malicious intent. If you do this, you put your entire database at risk because once a user finds out he or she can exploit your database they can do just about anything to it. Never ever put your arguments directly inside the conditions string.

TIP: For more information on the dangers of SQL injection, see the "Ruby on Rails Security Guide":security.html#sql-injection.

h5. Placeholder Conditions

Similar to the +(?)+ replacement style of params, you can also specify keys/values hash in your array conditions:

<ruby>
Client.where("created_at >= :start_date AND created_at <= :end_date",
  {:start_date => params[:start_date], :end_date => params[:end_date]})
</ruby>

This makes for clearer readability if you have a large number of variable conditions.

h5(#array-range_conditions). Range Conditions

If you're looking for a range inside of a table (for example, users created in a certain timeframe) you can use the conditions option coupled with the +IN+ SQL statement for this. If you had two dates coming in from a controller you could do something like this to look for a range:

<ruby>
Client.where(:created_at => (params[:start_date].to_date)..(params[:end_date].to_date))
</ruby>

This query will generate something similar to the following SQL:

<sql>
  SELECT "clients".* FROM "clients" WHERE ("clients"."created_at" BETWEEN '2010-09-29' AND '2010-11-30')
</sql>

h4. Hash Conditions

Active Record also allows you to pass in hash conditions which can increase the readability of your conditions syntax. With hash conditions, you pass in a hash with keys of the fields you want conditionalised and the values of how you want to conditionalise them:

NOTE: Only equality, range and subset checking are possible with Hash conditions.

h5. Equality Conditions

<ruby>
Client.where(:locked => true)
</ruby>

The field name can also be a string:

<ruby>
Client.where('locked' => true)
</ruby>

h5(#hash-range_conditions). Range Conditions

The good thing about this is that we can pass in a range for our fields without it generating a large query as shown in the preamble of this section.

<ruby>
Client.where(:created_at => (Time.now.midnight - 1.day)..Time.now.midnight)
</ruby>

This will find all clients created yesterday by using a +BETWEEN+ SQL statement:

<sql>
SELECT * FROM clients WHERE (clients.created_at BETWEEN '2008-12-21 00:00:00' AND '2008-12-22 00:00:00')
</sql>

This demonstrates a shorter syntax for the examples in "Array Conditions":#array-conditions

h5. Subset Conditions

If you want to find records using the +IN+ expression you can pass an array to the conditions hash:

<ruby>
Client.where(:orders_count => [1,3,5])
</ruby>

This code will generate SQL like this:

<sql>
SELECT * FROM clients WHERE (clients.orders_count IN (1,3,5))
</sql>

h3. Ordering

To retrieve records from the database in a specific order, you can use the +order+ method.

For example, if you're getting a set of records and want to order them in ascending order by the +created_at+ field in your table:

<ruby>
Client.order("created_at")
</ruby>

You could specify +ASC+ or +DESC+ as well:

<ruby>
Client.order("created_at DESC")
# OR
Client.order("created_at ASC")
</ruby>

Or ordering by multiple fields:

<ruby>
Client.order("orders_count ASC, created_at DESC")
</ruby>

h3. Selecting Specific Fields

By default, <tt>Model.find</tt> selects all the fields from the result set using +select *+.

To select only a subset of fields from the result set, you can specify the subset via the +select+ method.

NOTE: If the +select+ method is used, all the returning objects will be "read only":#readonly-objects.

<br />

For example, to select only +viewable_by+ and +locked+ columns:

<ruby>
Client.select("viewable_by, locked")
</ruby>

The SQL query used by this find call will be somewhat like:

<sql>
SELECT viewable_by, locked FROM clients
</sql>

Be careful because this also means you're initializing a model object with only the fields that you've selected. If you attempt to access a field that is not in the initialized record you'll receive:

<shell>
ActiveModel::MissingAttributeError: missing attribute: <attribute>
</shell>

Where +&lt;attribute&gt;+ is the attribute you asked for. The +id+ method will not raise the +ActiveRecord::MissingAttributeError+, so just be careful when working with associations because they need the +id+ method to function properly.

If you would like to only grab a single record per unique value in a certain field, you can use +uniq+:

<ruby>
Client.select(:name).uniq
</ruby>

This would generate SQL like:

<sql>
SELECT DISTINCT name FROM clients
</sql>

You can also remove the uniqueness constraint:

<ruby>
query = Client.select(:name).uniq
# => Returns unique names

query.uniq(false)
# => Returns all names, even if there are duplicates
</ruby>

h3. Limit and Offset

To apply +LIMIT+ to the SQL fired by the +Model.find+, you can specify the +LIMIT+ using +limit+ and +offset+ methods on the relation.

You can use +limit+ to specify the number of records to be retrieved, and use +offset+ to specify the number of records to skip before starting to return the records. For example

<ruby>
Client.limit(5)
</ruby>

will return a maximum of 5 clients and because it specifies no offset it will return the first 5 in the table. The SQL it executes looks like this:

<sql>
SELECT * FROM clients LIMIT 5
</sql>

Adding +offset+ to that

<ruby>
Client.limit(5).offset(30)
</ruby>

will return instead a maximum of 5 clients beginning with the 31st. The SQL looks like:

<sql>
SELECT * FROM clients LIMIT 5 OFFSET 30
</sql>

h3. Group

To apply a +GROUP BY+ clause to the SQL fired by the finder, you can specify the +group+ method on the find.

For example, if you want to find a collection of the dates orders were created on:

<ruby>
Order.select("date(created_at) as ordered_date, sum(price) as total_price").group("date(created_at)")
</ruby>

And this will give you a single +Order+ object for each date where there are orders in the database.

The SQL that would be executed would be something like this:

<sql>
SELECT date(created_at) as ordered_date, sum(price) as total_price FROM orders GROUP BY date(created_at)
</sql>

h3. Having

SQL uses the +HAVING+ clause to specify conditions on the +GROUP BY+ fields. You can add the +HAVING+ clause to the SQL fired by the +Model.find+ by adding the +:having+ option to the find.

For example:

<ruby>
Order.select("date(created_at) as ordered_date, sum(price) as total_price").group("date(created_at)").having("sum(price) > ?", 100)
</ruby>

The SQL that would be executed would be something like this:

<sql>
SELECT date(created_at) as ordered_date, sum(price) as total_price FROM orders GROUP BY date(created_at) HAVING sum(price) > 100
</sql>

This will return single order objects for each day, but only those that are ordered more than $100 in a day.

h3. Overriding Conditions

h4. +except+

You can specify certain conditions to be excepted by using the +except+ method. For example:

<ruby>
Post.where('id > 10').limit(20).order('id asc').except(:order)
</ruby>

The SQL that would be executed:

<sql>
SELECT * FROM posts WHERE id > 10 LIMIT 20
</sql>

h4. +only+

You can also override conditions using the +only+ method. For example:

<ruby>
Post.where('id > 10').limit(20).order('id desc').only(:order, :where)
</ruby>

The SQL that would be executed:

<sql>
SELECT * FROM posts WHERE id > 10 ORDER BY id DESC
</sql>

h4. +reorder+

The +reorder+ method overrides the default scope order. For example:

<ruby>
class Post < ActiveRecord::Base
  ..
  ..
  has_many :comments, :order => 'posted_at DESC'
end

Post.find(10).comments.reorder('name')
</ruby>

The SQL that would be executed:

<sql>
SELECT * FROM posts WHERE id = 10 ORDER BY name
</sql>

In case the +reorder+ clause is not used, the SQL executed would be:

<sql>
SELECT * FROM posts WHERE id = 10 ORDER BY posted_at DESC
</sql>

h4. +reverse_order+

The +reverse_order+ method reverses the ordering clause if specified.

<ruby>
Client.where("orders_count > 10").order(:name).reverse_order
</ruby>

The SQL that would be executed:

<sql>
SELECT * FROM clients WHERE orders_count > 10 ORDER BY name DESC
</sql>

If no ordering clause is specified in the query, the +reverse_order+ orders by the primary key in reverse order.

<ruby>
Client.where("orders_count > 10").reverse_order
</ruby>

The SQL that would be executed:

<sql>
SELECT * FROM clients WHERE orders_count > 10 ORDER BY clients.id DESC
</sql>

This method accepts *no* arguments.

h3. Readonly Objects

Active Record provides +readonly+ method on a relation to explicitly disallow modification or deletion of any of the returned object. Any attempt to alter or destroy a readonly record will not succeed, raising an +ActiveRecord::ReadOnlyRecord+ exception.

<ruby>
client = Client.readonly.first
client.visits += 1
client.save
</ruby>

As +client+ is explicitly set to be a readonly object, the above code will raise an +ActiveRecord::ReadOnlyRecord+ exception when calling +client.save+ with an updated value of _visits_.

h3. Locking Records for Update

Locking is helpful for preventing race conditions when updating records in the database and ensuring atomic updates.

Active Record provides two locking mechanisms:

* Optimistic Locking
* Pessimistic Locking

h4. Optimistic Locking

Optimistic locking allows multiple users to access the same record for edits, and assumes a minimum of conflicts with the data. It does this by checking whether another process has made changes to a record since it was opened. An +ActiveRecord::StaleObjectError+ exception is thrown if that has occurred and the update is ignored.

<strong>Optimistic locking column</strong>

In order to use optimistic locking, the table needs to have a column called +lock_version+. Each time the record is updated, Active Record increments the +lock_version+ column. If an update request is made with a lower value in the +lock_version+ field than is currently in the +lock_version+ column in the database, the update request will fail with an +ActiveRecord::StaleObjectError+. Example:

<ruby>
c1 = Client.find(1)
c2 = Client.find(1)

c1.first_name = "Michael"
c1.save

c2.name = "should fail"
c2.save # Raises an ActiveRecord::StaleObjectError
</ruby>

You're then responsible for dealing with the conflict by rescuing the exception and either rolling back, merging, or otherwise apply the business logic needed to resolve the conflict.

NOTE: You must ensure that your database schema defaults the +lock_version+ column to +0+.

This behavior can be turned off by setting <tt>ActiveRecord::Base.lock_optimistically = false</tt>.

To override the name of the +lock_version+ column, +ActiveRecord::Base+ provides a class method called +set_locking_column+:

<ruby>
class Client < ActiveRecord::Base
  set_locking_column :lock_client_column
end
</ruby>

h4. Pessimistic Locking

Pessimistic locking uses a locking mechanism provided by the underlying database. Using +lock+ when building a relation obtains an exclusive lock on the selected rows. Relations using +lock+ are usually wrapped inside a transaction for preventing deadlock conditions.

For example:

<ruby>
Item.transaction do
  i = Item.lock.first
  i.name = 'Jones'
  i.save
end
</ruby>

The above session produces the following SQL for a MySQL backend:

<sql>
SQL (0.2ms)   BEGIN
Item Load (0.3ms)   SELECT * FROM `items` LIMIT 1 FOR UPDATE
Item Update (0.4ms)   UPDATE `items` SET `updated_at` = '2009-02-07 18:05:56', `name` = 'Jones' WHERE `id` = 1
SQL (0.8ms)   COMMIT
</sql>

You can also pass raw SQL to the +lock+ method for allowing different types of locks. For example, MySQL has an expression called +LOCK IN SHARE MODE+ where you can lock a record but still allow other queries to read it. To specify this expression just pass it in as the lock option:

<ruby>
Item.transaction do
  i = Item.lock("LOCK IN SHARE MODE").find(1)
  i.increment!(:views)
end
</ruby>

If you already have an instance of your model, you can start a transaction and acquire the lock in one go using the following code:

<ruby>
item = Item.first
item.with_lock do
  # This block is called within a transaction,
  # item is already locked.
  item.increment!(:views)
end
</ruby>

h3. Joining Tables

Active Record provides a finder method called +joins+ for specifying +JOIN+ clauses on the resulting SQL. There are multiple ways to use the +joins+ method.

h4. Using a String SQL Fragment

You can just supply the raw SQL specifying the +JOIN+ clause to +joins+:

<ruby>
Client.joins('LEFT OUTER JOIN addresses ON addresses.client_id = clients.id')
</ruby>

This will result in the following SQL:

<sql>
SELECT clients.* FROM clients LEFT OUTER JOIN addresses ON addresses.client_id = clients.id
</sql>

h4. Using Array/Hash of Named Associations

WARNING: This method only works with +INNER JOIN+.

Active Record lets you use the names of the "associations":association_basics.html defined on the model as a shortcut for specifying +JOIN+ clause for those associations when using the +joins+ method.

For example, consider the following +Category+, +Post+, +Comments+ and +Guest+ models:

<ruby>
class Category < ActiveRecord::Base
  has_many :posts
end

class Post < ActiveRecord::Base
  belongs_to :category
  has_many :comments
  has_many :tags
end

class Comment < ActiveRecord::Base
  belongs_to :post
  has_one :guest
end

class Guest < ActiveRecord::Base
  belongs_to :comment
end

class Tag < ActiveRecord::Base
  belongs_to :post
end
</ruby>

Now all of the following will produce the expected join queries using +INNER JOIN+:

h5. Joining a Single Association

<ruby>
Category.joins(:posts)
</ruby>

This produces:

<sql>
SELECT categories.* FROM categories
  INNER JOIN posts ON posts.category_id = categories.id
</sql>

Or, in English: "return a Category object for all categories with posts". Note that you will see duplicate categories if more than one post has the same category. If you want unique categories, you can use Category.joins(:post).select("distinct(categories.id)").

h5. Joining Multiple Associations

<ruby>
Post.joins(:category, :comments)
</ruby>

This produces:

<sql>
SELECT posts.* FROM posts
  INNER JOIN categories ON posts.category_id = categories.id
  INNER JOIN comments ON comments.post_id = posts.id
</sql>

Or, in English: "return all posts that have a category and at least one comment". Note again that posts with multiple comments will show up multiple times.

h5. Joining Nested Associations (Single Level)

<ruby>
Post.joins(:comments => :guest)
</ruby>

This produces:

<sql>
SELECT posts.* FROM posts
  INNER JOIN comments ON comments.post_id = posts.id
  INNER JOIN guests ON guests.comment_id = comments.id
</sql>

Or, in English: "return all posts that have a comment made by a guest."

h5. Joining Nested Associations (Multiple Level)

<ruby>
Category.joins(:posts => [{:comments => :guest}, :tags])
</ruby>

This produces:

<sql>
SELECT categories.* FROM categories
  INNER JOIN posts ON posts.category_id = categories.id
  INNER JOIN comments ON comments.post_id = posts.id
  INNER JOIN guests ON guests.comment_id = comments.id
  INNER JOIN tags ON tags.post_id = posts.id
</sql>

h4. Specifying Conditions on the Joined Tables

You can specify conditions on the joined tables using the regular "Array":#array-conditions and "String":#pure-string-conditions conditions. "Hash conditions":#hash-conditions provides a special syntax for specifying conditions for the joined tables:

<ruby>
time_range = (Time.now.midnight - 1.day)..Time.now.midnight
Client.joins(:orders).where('orders.created_at' => time_range)
</ruby>

An alternative and cleaner syntax is to nest the hash conditions:

<ruby>
time_range = (Time.now.midnight - 1.day)..Time.now.midnight
Client.joins(:orders).where(:orders => {:created_at => time_range})
</ruby>

This will find all clients who have orders that were created yesterday, again using a +BETWEEN+ SQL expression.

h3. Eager Loading Associations

Eager loading is the mechanism for loading the associated records of the objects returned by +Model.find+ using as few queries as possible.

<strong>N <plus> 1 queries problem</strong>

Consider the following code, which finds 10 clients and prints their postcodes:

<ruby>
clients = Client.limit(10)

clients.each do |client|
  puts client.address.postcode
end
</ruby>

This code looks fine at the first sight. But the problem lies within the total number of queries executed. The above code executes 1 ( to find 10 clients ) <plus> 10 ( one per each client to load the address ) = <strong>11</strong> queries in total.

<strong>Solution to N <plus> 1 queries problem</strong>

Active Record lets you specify in advance all the associations that are going to be loaded. This is possible by specifying the +includes+ method of the +Model.find+ call. With +includes+, Active Record ensures that all of the specified associations are loaded using the minimum possible number of queries.

Revisiting the above case, we could rewrite +Client.all+ to use eager load addresses:

<ruby>
clients = Client.includes(:address).limit(10)

clients.each do |client|
  puts client.address.postcode
end
</ruby>

The above code will execute just <strong>2</strong> queries, as opposed to <strong>11</strong> queries in the previous case:

<sql>
SELECT * FROM clients LIMIT 10
SELECT addresses.* FROM addresses
  WHERE (addresses.client_id IN (1,2,3,4,5,6,7,8,9,10))
</sql>

h4. Eager Loading Multiple Associations

Active Record lets you eager load any number of associations with a single +Model.find+ call by using an array, hash, or a nested hash of array/hash with the +includes+ method.

h5. Array of Multiple Associations

<ruby>
Post.includes(:category, :comments)
</ruby>

This loads all the posts and the associated category and comments for each post.

h5. Nested Associations Hash

<ruby>
Category.includes(:posts => [{:comments => :guest}, :tags]).find(1)
</ruby>

This will find the category with id 1 and eager load all of the associated posts, the associated posts' tags and comments, and every comment's guest association.

h4. Specifying Conditions on Eager Loaded Associations

Even though Active Record lets you specify conditions on the eager loaded associations just like +joins+, the recommended way is to use "joins":#joining-tables instead.

However if you must do this, you may use +where+ as you would normally.

<ruby>
Post.includes(:comments).where("comments.visible", true)
</ruby>

This would generate a query which contains a +LEFT OUTER JOIN+ whereas the +joins+ method would generate one using the +INNER JOIN+ function instead.

<ruby>
  SELECT "posts"."id" AS t0_r0, ... "comments"."updated_at" AS t1_r5 FROM "posts" LEFT OUTER JOIN "comments" ON "comments"."post_id" = "posts"."id" WHERE (comments.visible = 1)
</ruby>

If there was no +where+ condition, this would generate the normal set of two queries.

If, in the case of this +includes+ query, there were no comments for any posts, all the posts would still be loaded. By using +joins+ (an INNER JOIN), the join conditions *must* match, otherwise no records will be returned.

h3. Scopes

Scoping allows you to specify commonly-used ARel queries which can be referenced as method calls on the association objects or models. With these scopes, you can use every method previously covered such as +where+, +joins+ and +includes+. All scope methods will return an +ActiveRecord::Relation+ object which will allow for further methods (such as other scopes) to be called on it.

To define a simple scope, we use the +scope+ method inside the class, passing the ARel query that we'd like run when this scope is called:

<ruby>
class Post < ActiveRecord::Base
  scope :published, where(:published => true)
end
</ruby>

Just like before, these methods are also chainable:

<ruby>
class Post < ActiveRecord::Base
  scope :published, where(:published => true).joins(:category)
end
</ruby>

Scopes are also chainable within scopes:

<ruby>
class Post < ActiveRecord::Base
  scope :published, where(:published => true)
  scope :published_and_commented, published.and(self.arel_table[:comments_count].gt(0))
end
</ruby>

To call this +published+ scope we can call it on either the class:

<ruby>
Post.published # => [published posts]
</ruby>

Or on an association consisting of +Post+ objects:

<ruby>
category = Category.first
category.posts.published # => [published posts belonging to this category]
</ruby>

h4. Working with times

If you're working with dates or times within scopes, due to how they are evaluated, you will need to use a lambda so that the scope is evaluated every time.

<ruby>
class Post < ActiveRecord::Base
  scope :last_week, lambda { where("created_at < ?", Time.zone.now ) }
end
</ruby>

Without the +lambda+, this +Time.zone.now+ will only be called once.

h4. Passing in arguments

When a +lambda+ is used for a +scope+, it can take arguments:

<ruby>
class Post < ActiveRecord::Base
  scope :1_week_before, lambda { |time| where("created_at < ?", time) }
end
</ruby>

This may then be called using this:

<ruby>
Post.1_week_before(Time.zone.now)
</ruby>

However, this is just duplicating the functionality that would be provided to you by a class method.

<ruby>
class Post < ActiveRecord::Base
  def self.1_week_before(time)
    where("created_at < ?", time)
  end
end
</ruby>

Using a class method is the preferred way to accept arguments for scopes. These methods will still be accessible on the association objects:

<ruby>
category.posts.1_week_before(time)
</ruby>

h4. Working with scopes

Where a relational object is required, the +scoped+ method may come in handy. This will return an +ActiveRecord::Relation+ object which can have further scoping applied to it afterwards. A place where this may come in handy is on associations

<ruby>
client = Client.find_by_first_name("Ryan")
orders = client.orders.scoped
</ruby>

With this new +orders+ object, we are able to ascertain that this object can have more scopes applied to it. For instance, if we wanted to return orders only in the last 30 days at a later point.

<ruby>
orders.where("created_at > ?", 30.days.ago)
</ruby>

h4. Applying a default scope

If we wish for a scope to be applied across all queries to the model we can use the +default_scope+ method within the model itself.

<ruby>
class Client < ActiveRecord::Base
  default_scope where("removed_at IS NULL")
end
</ruby>

When queries are executed on this model, the SQL query will now look something like this:

<sql>
SELECT * FROM clients WHERE removed_at IS NULL
</sql>

h4. Removing all scoping

If we wish to remove scoping for any reason we can use the +unscoped+ method. This is especially useful if a +default_scope+ is specified in the model and should not be applied for this particular query.

<ruby>
Client.unscoped.all
</ruby>

This method removes all scoping and will do a normal query on the table.

h3. Dynamic Finders

For every field (also known as an attribute) you define in your table, Active Record provides a finder method. If you have a field called +first_name+ on your +Client+ model for example, you get +find_by_first_name+ and +find_all_by_first_name+ for free from Active Record. If you have a +locked+ field on the +Client+ model, you also get +find_by_locked+ and +find_all_by_locked+ methods.

You can also use +find_last_by_*+ methods which will find the last record matching your argument.

You can specify an exclamation point (<tt>!</tt>) on the end of the dynamic finders to get them to raise an +ActiveRecord::RecordNotFound+ error if they do not return any records, like +Client.find_by_name!("Ryan")+

If you want to find both by name and locked, you can chain these finders together by simply typing "+and+" between the fields. For example, +Client.find_by_first_name_and_locked("Ryan", true)+.

WARNING: Up to and including Rails 3.1, when the number of arguments passed to a dynamic finder method is lesser than the number of fields, say <tt>Client.find_by_name_and_locked("Ryan")</tt>, the behavior is to pass +nil+ as the missing argument. This is *unintentional* and this behavior will be changed in Rails 3.2 to throw an +ArgumentError+.

h3. Find or build a new object

It's common that you need to find a record or create it if it doesn't exist. You can do that with the +first_or_create+ and +first_or_create!+ methods.

h4. +first_or_create+

The +first_or_create+ method checks whether +first+ returns +nil+ or not. If it does return +nil+, then +create+ is called. This is very powerful when coupled with the +where+ method. Let's see an example.

Suppose you want to find a client named 'Andy', and if there's none, create one and additionally set his +locked+ attribute to false. You can do so by running:

<ruby>
Client.where(:first_name => 'Andy').first_or_create(:locked => false)
# => #<Client id: 1, first_name: "Andy", orders_count: 0, locked: false, created_at: "2011-08-30 06:09:27", updated_at: "2011-08-30 06:09:27">
</ruby>

The SQL generated by this method looks like this:

<sql>
SELECT * FROM clients WHERE (clients.first_name = 'Andy') LIMIT 1
BEGIN
INSERT INTO clients (created_at, first_name, locked, orders_count, updated_at) VALUES ('2011-08-30 05:22:57', 'Andy', 0, NULL, '2011-08-30 05:22:57')
COMMIT
</sql>

+first_or_create+ returns either the record that already exists or the new record. In our case, we didn't already have a client named Andy so the record is created and returned.

The new record might not be saved to the database; that depends on whether validations passed or not (just like +create+).

It's also worth noting that +first_or_create+ takes into account the arguments of the +where+ method. In the example above we didn't explicitly pass a +:first_name => 'Andy'+ argument to +first_or_create+. However, that was used when creating the new record because it was already passed before to the +where+ method.

You can do the same with the +find_or_create_by+ method:

<ruby>
Client.find_or_create_by_first_name(:first_name => "Andy", :locked => false)
</ruby>

This method still works, but it's encouraged to use +first_or_create+ because it's more explicit on which arguments are used to _find_ the record and which are used to _create_, resulting in less confusion overall.

h4. +first_or_create!+

You can also use +first_or_create!+ to raise an exception if the new record is invalid. Validations are not covered on this guide, but let's assume for a moment that you temporarily add

<ruby>
validates :orders_count, :presence => true
</ruby>

to your +Client+ model. If you try to create a new +Client+ without passing an +orders_count+, the record will be invalid and an exception will be raised:

<ruby>
Client.where(:first_name => 'Andy').first_or_create!(:locked => false)
# => ActiveRecord::RecordInvalid: Validation failed: Orders count can't be blank
</ruby>

As with +first_or_create+ there is a +find_or_create_by!+ method but the +first_or_create!+ method is preferred for clarity.

h4. +first_or_initialize+

The +first_or_initialize+ method will work just like +first_or_create+ but it will not call +create+ but +new+. This means that a new model instance will be created in memory but won't be saved to the database. Continuing with the +first_or_create+ example, we now want the client named 'Nick':

<ruby>
nick = Client.where(:first_name => 'Nick').first_or_initialize(:locked => false)
# => <Client id: nil, first_name: "Nick", orders_count: 0, locked: false, created_at: "2011-08-30 06:09:27", updated_at: "2011-08-30 06:09:27">

nick.persisted?
# => false

nick.new_record?
# => true
</ruby>

Because the object is not yet stored in the database, the SQL generated looks like this:

<sql>
SELECT * FROM clients WHERE (clients.first_name = 'Nick') LIMIT 1
</sql>

When you want to save it to the database, just call +save+:

<ruby>
nick.save
# => true
</ruby>

h3. Finding by SQL

If you'd like to use your own SQL to find records in a table you can use +find_by_sql+. The +find_by_sql+ method will return an array of objects even if the underlying query returns just a single record. For example you could run this query:

<ruby>
Client.find_by_sql("SELECT * FROM clients
  INNER JOIN orders ON clients.id = orders.client_id
  ORDER clients.created_at desc")
</ruby>

+find_by_sql+ provides you with a simple way of making custom calls to the database and retrieving instantiated objects.

h3. +select_all+

<tt>find_by_sql</tt> has a close relative called +connection#select_all+. +select_all+ will retrieve objects from the database using custom SQL just like +find_by_sql+ but will not instantiate them. Instead, you will get an array of hashes where each hash indicates a record.

<ruby>
Client.connection.select_all("SELECT * FROM clients WHERE id = '1'")
</ruby>

h3. +pluck+

<tt>pluck</tt> can be used to query a single column from the underlying table of a model. It accepts a column name as argument and returns an array of values of the specified column with the corresponding data type.

<ruby>
Client.where(:active => true).pluck(:id)
# SELECT id FROM clients WHERE active = 1

Client.uniq.pluck(:role)
# SELECT DISTINCT role FROM clients
</ruby>

+pluck+ makes it possible to replace code like

<ruby>
Client.select(:id).map { |c| c.id }
</ruby>

with

<ruby>
Client.pluck(:id)
</ruby>

h3. Existence of Objects

If you simply want to check for the existence of the object there's a method called +exists?+. This method will query the database using the same query as +find+, but instead of returning an object or collection of objects it will return either +true+ or +false+.

<ruby>
Client.exists?(1)
</ruby>

The +exists?+ method also takes multiple ids, but the catch is that it will return true if any one of those records exists.

<ruby>
Client.exists?(1,2,3)
# or
Client.exists?([1,2,3])
</ruby>

It's even possible to use +exists?+ without any arguments on a model or a relation.

<ruby>
Client.where(:first_name => 'Ryan').exists?
</ruby>

The above returns +true+ if there is at least one client with the +first_name+ 'Ryan' and +false+ otherwise.

<ruby>
Client.exists?
</ruby>

The above returns +false+ if the +clients+ table is empty and +true+ otherwise.

You can also use +any?+ and +many?+ to check for existence on a model or relation.

<ruby>
# via a model
Post.any?
Post.many?

# via a named scope
Post.recent.any?
Post.recent.many?

# via a relation
Post.where(:published => true).any?
Post.where(:published => true).many?

# via an association
Post.first.categories.any?
Post.first.categories.many?
</ruby>

h3. Calculations

This section uses count as an example method in this preamble, but the options described apply to all sub-sections.

All calculation methods work directly on a model:

<ruby>
Client.count
# SELECT count(*) AS count_all FROM clients
</ruby>

Or on a relation:

<ruby>
Client.where(:first_name => 'Ryan').count
# SELECT count(*) AS count_all FROM clients WHERE (first_name = 'Ryan')
</ruby>

You can also use various finder methods on a relation for performing complex calculations:

<ruby>
Client.includes("orders").where(:first_name => 'Ryan', :orders => {:status => 'received'}).count
</ruby>

Which will execute:

<sql>
SELECT count(DISTINCT clients.id) AS count_all FROM clients
  LEFT OUTER JOIN orders ON orders.client_id = client.id WHERE
  (clients.first_name = 'Ryan' AND orders.status = 'received')
</sql>

h4. Count

If you want to see how many records are in your model's table you could call +Client.count+ and that will return the number. If you want to be more specific and find all the clients with their age present in the database you can use +Client.count(:age)+.

For options, please see the parent section, "Calculations":#calculations.

h4. Average

If you want to see the average of a certain number in one of your tables you can call the +average+ method on the class that relates to the table. This method call will look something like this:

<ruby>
Client.average("orders_count")
</ruby>

This will return a number (possibly a floating point number such as 3.14159265) representing the average value in the field.

For options, please see the parent section, "Calculations":#calculations.

h4. Minimum

If you want to find the minimum value of a field in your table you can call the +minimum+ method on the class that relates to the table. This method call will look something like this:

<ruby>
Client.minimum("age")
</ruby>

For options, please see the parent section, "Calculations":#calculations.

h4. Maximum

If you want to find the maximum value of a field in your table you can call the +maximum+ method on the class that relates to the table. This method call will look something like this:

<ruby>
Client.maximum("age")
</ruby>

For options, please see the parent section, "Calculations":#calculations.

h4. Sum

If you want to find the sum of a field for all records in your table you can call the +sum+ method on the class that relates to the table. This method call will look something like this:

<ruby>
Client.sum("orders_count")
</ruby>

For options, please see the parent section, "Calculations":#calculations.

h3. Running EXPLAIN

You can run EXPLAIN on the queries triggered by relations. For example,

<ruby>
User.where(:id => 1).joins(:posts).explain
</ruby>

may yield

<plain>
EXPLAIN for: SELECT `users`.* FROM `users` INNER JOIN `posts` ON `posts`.`user_id` = `users`.`id` WHERE `users`.`id` = 1
<plus>----<plus>-------------<plus>-------<plus>-------<plus>---------------<plus>---------<plus>---------<plus>-------<plus>------<plus>-------------<plus>
| id | select_type | table | type  | possible_keys | key     | key_len | ref   | rows | Extra       |
<plus>----<plus>-------------<plus>-------<plus>-------<plus>---------------<plus>---------<plus>---------<plus>-------<plus>------<plus>-------------<plus>
|  1 | SIMPLE      | users | const | PRIMARY       | PRIMARY | 4       | const |    1 |             |
|  1 | SIMPLE      | posts | ALL   | NULL          | NULL    | NULL    | NULL  |    1 | Using where |
<plus>----<plus>-------------<plus>-------<plus>-------<plus>---------------<plus>---------<plus>---------<plus>-------<plus>------<plus>-------------<plus>
2 rows in set (0.00 sec)
</plain>

under MySQL.

Active Record performs a pretty printing that emulates the one of the database
shells. So, the same query running with the PostgreSQL adapter would yield instead

<plain>
EXPLAIN for: SELECT "users".* FROM "users" INNER JOIN "posts" ON "posts"."user_id" = "users"."id" WHERE "users"."id" = 1
                                  QUERY PLAN
------------------------------------------------------------------------------
 Nested Loop Left Join  (cost=0.00..37.24 rows=8 width=0)
   Join Filter: (posts.user_id = users.id)
   ->  Index Scan using users_pkey on users  (cost=0.00..8.27 rows=1 width=4)
         Index Cond: (id = 1)
   ->  Seq Scan on posts  (cost=0.00..28.88 rows=8 width=4)
         Filter: (posts.user_id = 1)
(6 rows)
</plain>

Eager loading may trigger more than one query under the hood, and some queries
may need the results of previous ones. Because of that, +explain+ actually
executes the query, and then asks for the query plans. For example,

<ruby>
User.where(:id => 1).includes(:posts).explain
</ruby>

yields

<plain>
EXPLAIN for: SELECT `users`.* FROM `users`  WHERE `users`.`id` = 1
<plus>----<plus>-------------<plus>-------<plus>-------<plus>---------------<plus>---------<plus>---------<plus>-------<plus>------<plus>-------<plus>
| id | select_type | table | type  | possible_keys | key     | key_len | ref   | rows | Extra |
<plus>----<plus>-------------<plus>-------<plus>-------<plus>---------------<plus>---------<plus>---------<plus>-------<plus>------<plus>-------<plus>
|  1 | SIMPLE      | users | const | PRIMARY       | PRIMARY | 4       | const |    1 |       |
<plus>----<plus>-------------<plus>-------<plus>-------<plus>---------------<plus>---------<plus>---------<plus>-------<plus>------<plus>-------<plus>
1 row in set (0.00 sec)

EXPLAIN for: SELECT `posts`.* FROM `posts`  WHERE `posts`.`user_id` IN (1)
<plus>----<plus>-------------<plus>-------<plus>------<plus>---------------<plus>------<plus>---------<plus>------<plus>------<plus>-------------<plus>
| id | select_type | table | type | possible_keys | key  | key_len | ref  | rows | Extra       |
<plus>----<plus>-------------<plus>-------<plus>------<plus>---------------<plus>------<plus>---------<plus>------<plus>------<plus>-------------<plus>
|  1 | SIMPLE      | posts | ALL  | NULL          | NULL | NULL    | NULL |    1 | Using where |
<plus>----<plus>-------------<plus>-------<plus>------<plus>---------------<plus>------<plus>---------<plus>------<plus>------<plus>-------------<plus>
1 row in set (0.00 sec)
</plain>

under MySQL.

h4. Automatic EXPLAIN

Active Record is able to run EXPLAIN automatically on slow queries and log its
output. This feature is controlled by the configuration parameter

<ruby>
config.active_record.auto_explain_threshold_in_seconds
</ruby>

If set to a number, any query exceeding those many seconds will have its EXPLAIN
automatically triggered and logged. In the case of relations, the threshold is
compared to the total time needed to fetch records. So, a relation is seen as a
unit of work, no matter whether the implementation of eager loading involves
several queries under the hood.

A threshold of +nil+ disables automatic EXPLAINs.

The default threshold in development mode is 0.5 seconds, and +nil+ in test and
production modes.

INFO. Automatic EXPLAIN gets disabled if Active Record has no logger, regardless
of the value of the threshold.

h5. Disabling Automatic EXPLAIN

Automatic EXPLAIN can be selectively silenced with +ActiveRecord::Base.silence_auto_explain+:

<ruby>
ActiveRecord::Base.silence_auto_explain do
  # no automatic EXPLAIN is triggered here
end
</ruby>

That may be useful for queries you know are slow but fine, like a heavyweight
report of an admin interface.

As its name suggests, +silence_auto_explain+ only silences automatic EXPLAINs.
Explicit calls to +ActiveRecord::Relation#explain+ run.

h4. Interpreting EXPLAIN

Interpretation of the output of EXPLAIN is beyond the scope of this guide. The
following pointers may be helpful:

* SQLite3: "EXPLAIN QUERY PLAN":http://www.sqlite.org/eqp.html

* MySQL: "EXPLAIN Output Format":http://dev.mysql.com/doc/refman/5.6/en/explain-output.html

* PostgreSQL: "Using EXPLAIN":http://www.postgresql.org/docs/current/static/using-explain.html
